/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
//*****************************************************************************
// Development main.c for MSP430FR26xx_25xx
//
// This starter application initializes the CapTIvate touch library
// for the touch panel specified by CAPT_UserConfig.c/.h via a call to
// CAPT_appStart(), which initializes and calibrates all sensors in the
// application, and starts the CapTIvate interval timer.
//
// Then, the capacitive touch interface is driven by calling the CapTIvate
// application handler, CAPT_appHandler().  The application handler manages
// whether the user interface (UI) is running in full active scan mode, or
// in a low-power wake-on-proximity mode.
//
// The CapTIvate application handler will return true if proximity was
// detected on any of the sensors in the application, which is used here
// to control the state of LED2. LED1 is set while the background loop enters
// the handler, and is cleared when the background loop leaves the handler.
//
// \version 1.04.00.00
// Released on Mon, Aug 15, 2016  5:06:13 PM
//
//*****************************************************************************

#include <msp430.h>                      // Generic MSP430 Device Include
#include "driverlib.h"                   // MSPWare Driver Library
#include "captivate.h"                   // CapTIvate Touch Software Library
#include "CAPT_App.h"                    // CapTIvate Application Code
#include "CAPT_BSP.h"                    // CapTIvate EVM Board Support Package

#include <stdint.h>

// Definitions for the OPT3001 Light Level Sensor
// The only registers used in this example are the Result and Configuration Registers
#define OPT3001 0x44
#define Result 0x00
#define Configuration 0x01

// Definitions for the LP8863 LED Driver
// The only registers used in this example are the BL_MODE and BASE_BRT Registers
#define LP8863 0x2C
#define BL_MODE 0x20
#define BASE_BRT 0x28
#define ADAPT_BST 0x2AC

// Definitions for the TMP Sensor
// The only registers used in this example are the BL_MODE and BASE_BRT Registers
#define TMP423 0x4C // TMP423A 1001100b
#define LOCAL_MSB 0x00
#define LOCAL_LSB 0x10
#define REM1_MSB 0x01
#define REM1_LSB 0x11
#define REM2_MSB 0x02
#define REM2_LSB 0x12
#define REM3_MSB 0x03
#define REM3_LSB 0x13

// Definitions for number of bytes in an I2C transmission
#define ONE_BYTE 0x1
#define TWO_BYTES 0x2
#define THREE_BYTES 0x3

unsigned long lux;

// Function definition for the callback function to extract the current slider position value
void sliderPosition(tSensor* pSensor);

void GPIO_pin_configurations(void);
void LED_Driver_Enable(void);

//Definitions for the I2C related functions
void I2C_transmission_setup(unsigned char num_of_bytes);
void I2C_write(unsigned char slave_address, unsigned char register_address, unsigned char byte_1, unsigned char byte_2);
void I2C_setup_write(unsigned char slave_address, unsigned char register_address);
void I2C_read(unsigned char slave_address);


uint16_t current_position = 500;
bool appRunning;

unsigned int convert_lux_to_brightness(unsigned int lux_to_brightness);
unsigned long convert_to_lux(unsigned int exponent, unsigned int mantissa);

unsigned char TXData[3];
unsigned char RXData[2];
unsigned char ByteCtr;
unsigned char Local[2];
unsigned char RemoteOne[2];
unsigned char RemoteTwo[2];
unsigned char RemoteThree[2];
unsigned char test[2];

void main(void)
{
	//
	// Initialize the MCU
	// BSP_configureAlMCU() sets up the device IO and clocking
	// The global interrupt enable is set to allow peripherals
	// to wake the MCU.
	//
	WDTCTL = WDTPW | WDTHOLD;						// Stop watchdog timer
	//WDT_A_hold(WDT_A_BASE);
	//BSP_configureMCU();

	GPIO_pin_configurations();

	PM5CTL0 &= ~LOCKLPM5;                   		// Disable the GPIO power-on default high-impedance mode
	                                            	// to activate previously configured port settings

	LED_Driver_Enable();							// Enable the LED Driver to activate the boost circuitry

	//Write to Brightness Register (0x28) to change set initial brightness to 0
	//Writing a value of 0x0000 will set the brightness level to 0
	//I2C_transmission_setup(THREE_BYTES);
	//I2C_write(LP8863, BASE_BRT, 124,124);
	unsigned char exponent;
	unsigned int mantissa;

	unsigned int brightness = 0;
	unsigned int brightness_LSB;
	unsigned int brightness_MSB;
	unsigned int brightness_percent = 0;

	signed int adjustment = 0;

	//
	// Start the CapTIvate application
	//
	CAPT_appStart();

	//
	// Background Loop
	//


	while(1)
	{

    	//Write to Configuration Register (0x01) to set the operational mode of the OPT3001
    	//Writing a value of 0xC210 will configure the OPT3001 to complete a single shot
    	//conversion with a 100 ms conversion time; all other bits remain at default value
    	I2C_transmission_setup(THREE_BYTES);
    	I2C_write(OPT3001, Configuration, 0xC2, 0x10);

    	// Delay for 100 milliseconds to allow conversion process to complete
    	__delay_cycles(10000);

    	// Retreive temp data
        I2C_transmission_setup(ONE_BYTE);
        I2C_setup_write(TMP423, LOCAL_MSB);
        I2C_transmission_setup(ONE_BYTE);
        I2C_read(TMP423);
        Local[0] = RXData[0];
        I2C_transmission_setup(ONE_BYTE);
        I2C_setup_write(TMP423, LOCAL_LSB);
        I2C_transmission_setup(ONE_BYTE);
        I2C_read(TMP423);
        Local[1] = RXData[0];
        I2C_transmission_setup(ONE_BYTE);
        I2C_setup_write(TMP423, REM1_MSB);
        I2C_transmission_setup(ONE_BYTE);
        I2C_read(TMP423);
        RemoteOne[0] = RXData[0];
        I2C_transmission_setup(ONE_BYTE);
        I2C_setup_write(TMP423, REM2_MSB);
        I2C_transmission_setup(ONE_BYTE);
        I2C_read(TMP423);
        RemoteTwo[0] = RXData[0];
        I2C_transmission_setup(ONE_BYTE);
        I2C_setup_write(TMP423, REM3_MSB);
        I2C_transmission_setup(ONE_BYTE);
        I2C_read(TMP423);
        RemoteThree[0] = RXData[0];

    	//Setup Write to Result Register (0x00) of OPT3001 to prepare to read the the result of the conversion
    	//A setup write is required immediately prior to executing a read call
    	I2C_transmission_setup(ONE_BYTE);
    	I2C_setup_write(OPT3001, Result);

    	//Read of Result Register (0x00) of the OPT3001 to extract the result of the of the conversion
    	//The previous setup write specifies that the Result register will be read and sent to the MSP430
    	//The OPT3001 will retain the register value from the setup write when the read function is called
    	I2C_transmission_setup(TWO_BYTES);
    	I2C_read(OPT3001);

    	//Extract the upper four bits of exponent and the lower 12 bits of mantissa from the raw result value
    	//The Result value must be converted into a meaningful value
       	exponent = RXData[0] >> 4;
    	mantissa = (RXData[0] << 8 | RXData[1]) & 0x0FFF;

    	//Take the exponent and mantissa values and calculate the corresponding lux value
    	lux = convert_to_lux(exponent, mantissa);

    	//Convert the lux measurment into a brightness value and split it into the upper and lower bytes
    	//The current_position multiplier can be adjusted as necessary to help highlight the changes in
    	//the sensor position variable
    	brightness = convert_lux_to_brightness(lux);

    	brightness_percent = brightness * 0.2;

    	//Run the capTIvate application handler and extract the slider position
		appRunning = CAPT_appHandler();
		MAP_CAPT_registerCallback(&SLD00, &sliderPosition);
		//
		// End of background loop iteration
		// Go to sleep if there is nothing left to do
		//
		CAPT_appSleep();

		__delay_cycles(10000);

    	adjustment = (current_position - 500)/500*brightness_percent;

    	if(adjustment < 0 && brightness < (signed int)(adjustment*-1)){ //adjustment -1000 to 1000
    	    brightness = 0; // Changed 10X multiply to 1X
    	}
    	else
    	{
    	    brightness += adjustment; // (current_position*2);
    	}

    	brightness_LSB = brightness & 0x00FF;
    	brightness_MSB = brightness >> 8;

    	P1OUT |= BIT0;
    	//Delay 100 ms to prepare to send brightness value to LP8863 LED Driver
    	__delay_cycles(10000);

    	//Write to BASE_BRT Register (0x28) to change the brightness level
    	//Writing the converted value to the Brightness register will adjust the brightness level
    	//accordingly based on the conversion result from the light sensor
    	I2C_transmission_setup(THREE_BYTES);
        I2C_write(LP8863, BASE_BRT, brightness_LSB, brightness_MSB);

		__no_operation();

		

	} // End background loop
} // End main()


#if defined(__TI_COMPILER_VERSION__) || defined(__IAR_SYSTEMS_ICC__)
#pragma vector = USCI_B0_VECTOR
__interrupt void USCIB0_ISR(void)
#elif defined(__GNUC__)
void __attribute__ ((interrupt(USCI_B0_VECTOR))) USCIB0_ISR (void)
#else
#error Compiler not supported!
#endif
{
  switch(__even_in_range(UCB0IV,USCI_I2C_UCBIT9IFG))
  {
        case USCI_NONE: break;                        // Vector 0: No interrupts break;
        case USCI_I2C_UCALIFG: break;
        case USCI_I2C_UCNACKIFG:
            UCB0CTL1 |= UCTXSTT;                      //resend start if NACK
          break;                                      // Vector 4: NACKIFG break;
        case USCI_I2C_UCSTTIFG: break;                // Vector 6: STTIFG break;
        case USCI_I2C_UCSTPIFG: break;                // Vector 8: STPIFG break;
        case USCI_I2C_UCRXIFG3: break;                // Vector 10: RXIFG3 break;
        case USCI_I2C_UCTXIFG3: break;                // Vector 14: TXIFG3 break;
        case USCI_I2C_UCRXIFG2: break;                // Vector 16: RXIFG2 break;
        case USCI_I2C_UCTXIFG2: break;                // Vector 18: TXIFG2 break;
        case USCI_I2C_UCRXIFG1: break;                // Vector 20: RXIFG1 break;
        case USCI_I2C_UCTXIFG1: break;                // Vector 22: TXIFG1 break;

        case USCI_I2C_UCRXIFG0:                 	  // Vector 24: RXIFG0 break;

        	RXData[ByteCtr] = UCB0RXBUF;        		// Load RX buffer
        	ByteCtr++;									// Increment RX byte counter

			break;

        case USCI_I2C_UCTXIFG0:						  // Vector 26: TXIFG0 break;

        	UCB0TXBUF = TXData[ByteCtr];            // Load TX buffer
        	ByteCtr++;                              // Increment TX byte counter

          break;

        case USCI_I2C_UCBCNTIFG: 					  // Vector 28: BCNTIFG

        	__bic_SR_register_on_exit(LPM0_bits);     // Exit LPM0

        	break;

        case USCI_I2C_UCCLTOIFG: break;               // Vector 30: clock low timeout
        case USCI_I2C_UCBIT9IFG: break;               // Vector 32: 9th bit
        default: break;
  }
}

/*
	This function implements a call back function that will extract the Slider Position value from the capTIvate
	application handler and pass it into the main.c and make it accessible
	This function filters out values when the sensor is not being pressed as the capTIvate handler gives a result of
	0xFFFF when it detects no touch
	This software filtering allows the main.c program to retain the most recent pressed value on the slider while throwing
	out the values when the slider is not being touched
	The SliderPosition value can be used to adjust the brightness value in conjunction with the OPT3001 Light Sensor
*/
void sliderPosition(tSensor* pSensor)
{
	uint16_t SliderPosition;

	SliderPosition = ((tSliderSensorParams*)(pSensor->pSensorParams))->SliderPosition.ui16Natural;

	if(SliderPosition != 0xFFFF)
	{
		current_position = SliderPosition;
	}

}

/*
	In this example, the MSP430FR2633 needs to operate as both a transmitter and receiver
	In order for this to happen, every single I2C TX or RX fucntion call must be preceded by a transmission setup call
	This call will setup the I2C module to issue an automatic hardware stop command after a certain number of bytes
	have been either transmitted or received by comparing to the UCB0TBCNT register
	This function specifies the number of bytes in the upcoming transmission
	A write call will require 3 bytes
	A setup write will require 2 bytes
	A read will require 1 byte
*/
void I2C_transmission_setup(unsigned char num_of_bytes)
{
	//Configure USCI_B0 for I2C Mode and designate the number of bytes to be transmitted/received

    UCB0CTLW0 |= UCSWRST;                   			// Software reset enabled
	UCB0CTLW0 |= UCMODE_3 | UCMST | UCSYNC;            	// I2C master mode, SMCLK
	UCB0CTLW1 |= UCASTP_2;                  			// Automatic stop generated
	                                            		// after UCB0TBCNT is reached
	UCB0BRW = 0x8;                                  	// baudrate = SMCLK / 8
    UCB0TBCNT = num_of_bytes;              				// number of bytes to be received
    UCB0CTL1 &= ~UCSWRST;								// clear reset register
    UCB0IE |= UCTXIE0 | UCRXIE0 | UCNACKIE | UCBCNTIE;   // transmit and NACK interrupt enable
}

/*
	This function call will setup the I2C module to transmit 3 bytes of data from the MSP430 to
	one of the two slave devices on this board, either the OPT3001 or the LP8863
	The required parameters are the slave address, the register address, and then the LSB and MSB
	The actual registers are 16 bits wide but the I2C module can only transmit 8 bits at a time
	As a result, the transmitted value must be split into the MSB and LSB
*/
void I2C_write(unsigned char slave_address, unsigned char register_address, unsigned char byte_1, unsigned char byte_2)
{
    UCB0I2CSA = slave_address;						// Set the slave address
    TXData[0] = register_address;					// Set the control register address
    TXData[1] = byte_1;								// Set the Register Data LSB
    TXData[2] = byte_2;								// Set the Register Data MSB
    ByteCtr = 0;                                  	// Load byte counter
    while (UCB0CTLW0 & UCTXSTP);                    // Ensure stop condition got sent
    UCB0CTLW0 |= UCTR;								// I2C TX
    UCB0CTLW0 |= UCTXSTT;                    		// I2C start condition
    __bis_SR_register(LPM0_bits | GIE);             // Enter LPM0 w/ interrupts
    	                                            // Remain in LPM0 until all data
    												// is TX'd
}

/*
	This function call will setup the I2C module to transmit 2 bytes of data from the MSP430 to
	one of the two slave devices on this board, either the OPT3001 or the LP8863
	The required parameters are the slave address and the register address
	This function is used immediately before calling the I2C_read() function
	The I2C standard requires a write be issued to the slave address with the specified register
	and then followed with a read command
*/
void I2C_setup_write(unsigned char slave_address, unsigned char register_address)
{
    UCB0I2CSA = slave_address;						// Set the slave address
    TXData[0] = register_address;					// Set the control register address
    ByteCtr = 0;                                  	// Load byte counter
    while (UCB0CTLW0 & UCTXSTP);                    // Ensure stop condition got sent
    UCB0CTLW0 |= UCTR;								// I2C TX
    UCB0CTLW0 |= UCTXSTT;                    		// I2C start condition
    __bis_SR_register(LPM0_bits | GIE);             // Enter LPM0 w/ interrupts
    	                                            // Remain in LPM0 until all data
    												// is TX'd
}

/*
	This function call will setup the I2C module to receive 2 bytes of data from
	one of the two slave devices on this board, either the OPT3001 or the LP8863,
	to the MSP430
	The required parameter is the slave address
	This function is used immediately after calling the I2C_read() function
	The I2C standard requires a write be issued to the slave address with the specified register
	and then followed with a read command
	The result is sent to the RXData array and can be accessed by the MSP430
*/
void I2C_read(unsigned char slave_address)
{
	UCB0I2CSA = slave_address;						// Set the slave address
	ByteCtr = 0;                                  	// Load byte counter
	while (UCB0CTLW0 & UCTXSTP);                    // Ensure stop condition got sent
	UCB0CTLW0 &= ~UCTR;								// I2C RX
	UCB0CTLW0 |= UCTXSTT;                    		// I2C start condition
	__bis_SR_register(LPM0_bits | GIE);             // Enter LPM0 w/ interrupts
	    	    	                                // Remain in LPM0 until all data
	    	    									// is RX'd
}

/*
    This function will take the exponent and mantissa values from the OPT3001 Result register
    and convert it into a lux value based on a formula below
    This function implements a exponent function that does not require the math.h library
    as it creates too much overhead
*/
unsigned long convert_to_lux(unsigned int exponent, unsigned int mantissa)
{
    // Convert the Result Register into a lux measurement based on formula below:
    // lux = 0.01 * (2^E[3:0]) * R[11:0]

    unsigned int result;
    unsigned int count;
    unsigned long converted_lux;

    result = 1;
    for(count = exponent; count > 0; count--)
    {
        result = result * 2;
    }
    converted_lux = mantissa / 100;
    converted_lux *= result;

    return converted_lux;
}

/*
	This function will take converted lux values and assign an arbitrary brightness
	value based on the lux value
	This function will need to be modified based on the operating environment and conditions
	An external light meter should be used to establish the average brightness level
	This function should be used as a starting point and then tweaked for inidividual applications
*/
unsigned int convert_lux_to_brightness(unsigned int lux)
{
    int i;
    long dx, dy;
    static const unsigned int luxCode[12]=
    {
     65000, 20000, 7500,3000,700,300,125,30,5,0
    };
    static const unsigned int brightLvl[12]=
    {
     65000, 20000, 7500,3000,700,300,125,30,5,0
    };
    static const int count = 12;

    if (lux > luxCode[0]) {
        /* lux is greater than the max element
         * handle error here if you want */
        return brightLvl[0]; /* return minimum element */
    }

    if (lux < luxCode[count-1]) {
        return brightLvl[count-1]; /* return maximum */
    }

    /* find i, such that bitCode[i] <= x < bitCode[i-1] */
    for (i = count-1; i > 0; i--) {
        if (luxCode[i-1] > lux) {
            break;
        }
    }

    /* interpolate */
    dx = luxCode[i-1] - luxCode[i];
    dy = brightLvl[i-1] - brightLvl[i];
    return brightLvl[i] + (lux - luxCode[i]) * dy / dx;
}

void GPIO_pin_configurations(void)
{
    //Beginning of pin configurations

	//P2.0 is tied directly to the LDO that powers the LP8863
    P2OUT &= ~BIT0;                         		// Clear P2.0 output latch for a defined power-on state
    P2DIR |= BIT0;                          		// Set P2.0 to output direction

    //Set P1.6 as an input GPIO pin since it is tied to P1.1
    P1OUT &= ~BIT6;                         		// Clear P1.6 output latch for a defined power-on state
    P1DIR &= ~BIT6;                         		// Set P1.6 to input direction
    P1SEL1 &= ~BIT6;								// P1.6 options select

    //Set P1.2 and P1.3 pins for I2C functionality
    P1SEL0 |= BIT2 | BIT3;


    P1OUT &= ~BIT0;                                 // Clear P2.0 output latch for a defined power-on state
    P1DIR |= BIT0;
    P1REN |= BIT0;
    P1OUT &= ~BIT0;

    //End of pin configurations
}

void LED_Driver_Enable(void)
{
        P2OUT |= BIT0;									//Set P1.0 to enable LDO on LED Driver LP8863

        P1OUT |= BIT0;

		__delay_cycles(100000);

		P1OUT &= ~BIT0;
		__delay_cycles(150000);

		//Above toggles EN and drains residual voltage
		P2OUT &= ~BIT0;
		__delay_cycles(50000);
		P2OUT |= BIT0;
		__delay_cycles(10000);

		__bis_SR_register(GIE);

		//Write to BL_MODE Register (0x20) to change brightness_mode to I2C Control
		//Writing a value of 0x0202 will configure the LED Driver to control the brightness level
		//through the BASE_BRT I2C register
		I2C_transmission_setup(THREE_BYTES);
		I2C_write(LP8863, BL_MODE, 0x02, 0x02);
}

